use based::request::api::{api_error, to_uuid, vec_to_api, FallibleApiResponse, ToAPI};
use based::request::assets::DataResponse;
use rocket::tokio::io::AsyncReadExt;
use rocket::{get, tokio, State};

use crate::cache::RouteCache;
use crate::library::artist::Artist;
use crate::library::Libary;

/// Get all artists
#[get("/artists")]
pub async fn artists_route(lib: &State<Libary>) -> FallibleApiResponse {
    let artists = lib.get_artists().await;
    Ok(serde_json::to_value(&vec_to_api(&artists).await).unwrap())
}

#[get("/artist/<id>/image")]
pub async fn artist_image_route(id: &str, cache: &State<RouteCache>) -> Option<DataResponse> {
    let image = cache
        .get_option("artist_image_route", id, || async {
            Artist::get_image_of(&to_uuid(id).ok()?).await
        })
        .await?;

    Some(DataResponse::new_file(
        &image,
        "image/png".to_string(),
        Some(60 * 60 * 24 * 5),
    ))
}

#[get("/artist/<id>")]
pub async fn artist_route(id: &str) -> FallibleApiResponse {
    Ok(Artist::get(&to_uuid(id)?)
        .await
        .ok_or_else(|| api_error("No artist with that ID found"))?
        .api()
        .await)
}
